from flask_wtf import FlaskForm
from wtforms import FileField, DateField, StringField, BooleanField
from wtforms.validators import DataRequired, Optional


class UploadForm(FlaskForm):  # type: ignore
    filename = FileField('CSV file:', render_kw={
        'class': 'form-control',
        'aria-describedby': 'fileHelp'
    }, validators=[DataRequired()])
    config = FileField('Config file:', render_kw={
        'class': 'form-control',
        'aria-describedby': 'configHelp'
    }, validators=[Optional()])
    start_date = DateField('Start Date:', format='%Y-%m-%d', render_kw={
        'class': 'form-control',
        'id': 'start_date',
        'aria-describedby': 'dateStartHelp'
    }, validators=[Optional()])
    end_date = DateField('End Date:', format='%Y-%m-%d', render_kw={
        'class': 'form-control',
        'id': 'end_date',
        'aria-describedby': 'dateEndHelp'
    }, validators=[Optional()])
    filter = StringField('Filter:', render_kw={
        'class': 'form-control',
        'aria-describedby': 'filterHelp'
    })
    verbose = BooleanField('Verbose logs', render_kw={
        'class': 'form-check-input',
        'aria-describedby': 'verboseHelp'
    })
    no_currency_format = BooleanField('No Currency Format', render_kw={
        'class': 'form-check-input',
        'aria-describedby': 'currencyHelp'
    })



import pytest
from whatsthedamage.models.csv_row import CsvRow
from whatsthedamage.config.config import AppConfig, CsvConfig, AppContext
from whatsthedamage.config.config import AppArgs


@pytest.fixture
def mapping():
    return {
        'date': 'date',
        'type': 'type',
        'partner': 'partner',
        'amount': 'amount',
        'currency': 'currency'
    }


@pytest.fixture
def csv_rows(mapping):
    return [
        CsvRow(
            {'date': "2023-01-01", 'type': 'deposit', 'partner': 'bank', 'amount': 100, 'currency': 'EUR'},
            mapping),
        CsvRow(
            {'date': "2023-01-02", 'type': 'deposit', 'partner': 'bank', 'amount': 200, 'currency': 'EUR'},
            mapping),
    ]


@pytest.fixture
def app_context():
    # Create the CsvConfig object
    csv_config = CsvConfig(
        dialect="excel",
        delimiter=",",
        date_attribute_format="%Y-%m-%d",
        attribute_mapping={"date": "date", "amount": "amount"},
    )

    # Define enricher pattern sets
    enricher_pattern_sets = {
        "category1": {
            "pattern1": ["value1", "value2"],
            "pattern2": ["value3", "value4"]
        }
    }

    # Create the AppConfig object
    app_config = AppConfig(
        csv=csv_config,
        enricher_pattern_sets=enricher_pattern_sets
    )

    # Create the AppArgs object
    app_args = AppArgs(
        category="category1",
        config="config.json",
        end_date="2023-12-31",
        filename="data.csv",
        filter="filter_criteria",
        no_currency_format=False,
        nowrap=False,
        output_format="json",
        output=None,
        start_date="2023-01-01",
        verbose=True
    )

    # Return the AppContext object
    return AppContext(config=app_config, args=app_args)



from flask import Flask
import os
from whatsthedamage.controllers.routes import bp as main_bp
from typing import Optional


class AppConfig:
    UPLOAD_FOLDER: str = 'uploads'
    MAX_CONTENT_LENGTH: int = 16 * 1024 * 1024  # 16 MB
    SECRET_KEY: bytes = os.urandom(24)
    DEFAULT_WHATSTHEDAMAGE_CONFIG: str = 'config.yml.default'


def create_app(config_class: Optional[AppConfig] = None) -> Flask:
    app: Flask = Flask(__name__, template_folder='view/templates')

    # Load default configuration from a class
    app.config.from_object(AppConfig)

    if config_class:
        app.config.from_object(config_class)

    # Check if external config file exists and load it
    config_file = 'config.py'
    if os.path.exists(config_file):
        app.config.from_pyfile(config_file)

    # Ensure the upload folder exists
    os.makedirs(app.config['UPLOAD_FOLDER'], exist_ok=True)

    app.register_blueprint(main_bp)

    return app


if __name__ == '__main__':
    app = create_app(None)
    app.run(debug=True)



from typing import Dict, List
from whatsthedamage.models.csv_row import CsvRow


def print_categorized_rows(set_name: str, rows_dict: Dict[str, List[CsvRow]]) -> None:
    """
    Prints categorized rows from a dictionary.

    Args:
        set_name (str): The name of the set to be printed.
        rows_dict (Dict[str, List[CsvRow]]): A dictionary of type values and lists of CsvRow objects.

    Returns:
        None
    """
    print(f"\nSet name: {set_name}")
    for type_value, rowset in rows_dict.items():
        print(f"\nType: {type_value}")
        for row in rowset:
            print(repr(row))
